/*  zweic -- a compiler for Zwei
 *
 *  Stephane Micheloud & LAMP
 *
 *  $Id: AnalyzerTest.java 2252 2005-11-19 18:53:20 +0100 (Sat, 19 Nov 2005) michelou $
 */

package zweic;

import java.io.*;


/** A small test class for the semantic analyzer of zweic; it reads in a
 *  file and prints out the abstract syntax tree. If the program is
 *  erroneous, the compiler will print a corresponding error message
 *  instead.
 *
 *  usage: java zweic.AnalyzerTest <file.zwei>
 */
public class AnalyzerTest {
    public static void main(String[] args) {
        if (args.length != 1) {
            System.err.println("usage: java zweic.AnalyzerTest <file.zwei>");
            System.exit(1);
        }
        try {
            InputStreamReader in = new FileReader(args[0]);
            Tree tree = new Parser(in).parse();
            in.close();
            if (Report.errors > 0)
                System.err.println(Report.errors + " errors found." );
            else
                new Analyzer().analyze(tree);
            System.exit((Report.errors > 0) ? 1 : 0);
        }
        catch (IOException e) {
            Report.fail(e.getMessage());
        }
    }
}
