/*  zweic -- a compiler for Zwei
 *
 *  Stephane Micheloud & LAMP
 *
 *  $Id: Type.java 2239 2005-11-18 16:39:33 +0100 (Fri, 18 Nov 2005) michelou $
 */

package zweic;

import zweic.Symbol.*;


/**
 * This class provides a representation for types in zweic
 */
abstract class Type {

    /** This type represents an erroneous type (caused by a typing
     *  error).
     */
    public static final Type IBadType = new Type(){};

    /** The type of the null object reference.
     */
    public static final Type INullType = new Type(){};

    /** Type Int.
     */
    public static final Type IIntType = new Type(){};

    /**
     * This method checks if the provided type is a super type
     * of the receiver.
     */
    public boolean isSubtype(Type that) {
        return (this == IBadType)
        // ...  complter ...
                ;
    }

    /**
     * This method can be used to check if two types are equal. It
     * must be overriden for object types and function types. It will
     * not be used for class types.
     */
    public boolean isSameType(Type that) {
        return (this == IBadType)
        // ...  complter ...
                ;
    }

    public Type lub(Type that) {
        if      (this == IBadType) return that;
        // ...  complter ...
        else return null;
    }
  
    /**
     * Returns a textual representation of this type.
     */
    public String toString() {
        if (this == IBadType)  return "<bad>";
        if (this == IIntType)  return "Int";
        if (this == INullType) return "Null";
        throw new Error();
    }

    public static class IClassType extends Type {
        private ClassSymbol c;
    
        IClassType(ClassSymbol c) {
            this.c = c;
        }

        public ClassSymbol getClassSymbol() { return c; }

        public String toString() { return c.name; }
    }

}
